/* PICLAB.H : Constants, types, and globals for PICLAB are defined.  This
** file is included by all modules, so initializers are enclosed in "#ifdef
** MAIN_MODULE" blocks.
*/

#include <limits.h>
#include <setjmp.h>

#ifndef UTYPEDEFS
    #include "typedefs"
#endif

/* #define TEST    1 */

#define UTYPE   0       /* Used in vartable[]       */
#define FTYPE   1
#define STYPE   2
#define BTYPE   3
#define LTYPE   4
#define STRSIZE 32
#define BUFSIZE 16384
#define NFILES  6       /* Max number of open files */
#define NIMAGES 3       /* number of edit buffers   */
#define READ    1
#define WRITE   2

GLOBAL struct _ioblk {                  /* This structure and the following */
    int handle, width, mode;            /* prototypes are used in PLFILES.C */
    U16 bsize, bufp, bfull;
    U8 *iobuf, *linebuf;
    char fname[65];
} ioblocks[NFILES];

GLOBAL struct _imgbuf {
    char *bufname, *filename;
    U8 flags, planes;
    U16 width, height;
} images[NIMAGES], *old, *new, *itmp;

GLOBAL struct _plane {
    U8 *linebuf;
    int handle;
    U16 width;
} planes[NFILES];

char *makepath(char *, char *, char *);
int p_closeall(void);
int p_open(char *, char *, int, int, int);
int p_reopen(int, int);
int p_seek(int, long);
void p_close(int);
int p_getc(int);
int p_getline(int, int);
int p_putline(int, int);
struct _plane *openplane(int, struct _imgbuf *, int);
void closeplane(struct _plane *);
int getline(struct _plane *);
int putline(struct _plane *);
int seekline(struct _plane *, int);
int begintransform(void);

#define linebuf(h) ioblocks[h].linebuf

char *talloc(U16);              /* Memory allocation functions in MEMORY.C  */
U32 mark(void);
void release(U32);
void releaseall(void);
U32 freemem(void);

GLOBAL U8 *rv, *gv, *bv, *table2;   /* Interface to routines in PLDITHER.C  */
GLOBAL int startdither INIT(1);
void ditherline(U16, U8 *[3], U8 *);
void bitmapline(U16, U8 *, U8 *[2]);
void scaleline(U8 *, U8 *, int, int);

typedef struct _arg {           /* Structure passed to command-handlers     */
    char *cval;
    float fval;
} argument;

int addimg(int, argument *);        /* Command-handling routines.  These    */
int average(int, argument *);       /* should all take a count of arguments */
int brightness(int, argument *);    /* and a pointer to an array of them.   */
int clip(int, argument *);          /* To add a command to PICLAB, write a  */
int color(int, argument *);         /* function using these as a guide, put */
int contrast(int, argument *);      /* a prototype here, and add an entry   */
int expand(int, argument *);        /* to cmdtable[], below.                */
int filedir(int, argument *);
int gamma(int, argument *);
int gifdir(int, argument *);
int grayscale(int, argument *);
int histogram(int, argument *);
int list(int, argument *);
int load(int, argument *);
int loadgif(int, argument *);
int loadtga(int, argument *);
int loadrix(int, argument *);
int loadraw(int, argument *);
int loadip(int, argument *);
int median(int, argument *);
int mirror(int, argument *);
int negate(int, argument *);
int overlay(int, argument *);
int pause(int, argument *);
int printpic(int, argument *);
int quit(int, argument *);
int rescale(int, argument *);
int reverse(int, argument *);
int rotate(int, argument *);
int run(int, argument *);
int save(int, argument *);
int savegif(int, argument *);
int savetga(int, argument *);
int saverix(int, argument *);
int saveraw(int, argument *);
int saveip(int, argument *);
int setvar(int, argument *);
int sharpen(int, argument *);
int shell(int, argument *);
int showpic(int, argument *);
int smooth(int, argument *);
int subimg(int, argument *);
#ifdef TEST
  void testfunc(int, argument *);
#endif
int tgadir(int, argument *);
int transform(int, argument *);
int undo(int, argument *);

int cgamode(void);              /* Video adapter- and mode-dependent        */
void cgaline(int, U8 *[3]);     /* routines from PLSHOW.C.                  */
void cgastop(void);
int egacmode(void);
int vgacmode(void);
int svgamode(void);
void egacline(int, U8 *[3]);
void svgaline(int, U8 *[3]);
void egacstop(void);
int gray1mode(void);
void gray1line(int, U8 *[3]);
int gray2mode(void);
void gray2line(int, U8 *[3]);
void graystop(void);

void pl_warn(int);                          /* General-purpose routines     */
void pl_printf(char *, ...);                /* from PLMAIN.C                */
void pl_init(void);
void pl_trace(register int);
void pl_cleanup(void);
int pl_match(char *, char *[]);
int pl_parse(char *);
void main(int argc, char *argv[]);

GLOBAL jmp_buf interp;

GLOBAL struct {                         /* This table is used by parse() to */
    char *command;                      /* dispatch commands to the correct */
    int (*function)(int, argument *);   /* function.  Each is responsible   */
} cmdtable[]                            /* for interpreting the arguments   */
                                        /* passed to it; see the individual */
#ifdef MAIN_MODULE                      /* source files for more details.   */
    = { { "ADD",        addimg      },
        { "AVERAGE",    average     },
        { "BRIGHTEN",   brightness  },
        { "CALL",       shell       },
        { "CANCEL",     undo        },
        { "CLIP",       clip        },
        { "COLOR",      color       },
        { "CONTRAST",   contrast    },
        { "DARKEN",     brightness  },
        { "DIR",        filedir     },
        { "DOS",        shell       },
        { "EXIT",       quit        },
        { "EXPAND",     expand      },
        { "GAMMA",      gamma       },
        { "GDIR",       gifdir      },
        { "GLOAD",      loadgif     },
        { "GRAY",       grayscale   },
        { "GSAVE",      savegif     },
        { "HISTOGRAM",  histogram   },
        { "LIST",       list        },
        { "LOAD",       load        },
        { "MEDIAN",     median      },
        { "MIRROR",     mirror      },
        { "NEGATE",     negate      },
        { "OVERLAY",    overlay     },
        { "PAUSE",      pause       },
        { "PRINT",      printpic    },
        { "QUIT",       quit        },
        { "RESCALE",    rescale     },
        { "REVERSE",    reverse     },
        { "ROTATE",     rotate      },
        { "RUN",        run         },
        { "SAVE",       save        },
        { "SET",        setvar      },
        { "SHARPEN",    sharpen     },
        { "SHELL",      shell       },
        { "SHOW",       showpic     },
        { "SMOOTH",     smooth      },
        { "SUBTRACT",   subimg      },
  #ifdef TEST
        { "TEST",       testfunc    },
  #endif
        { "TDIR",       tgadir      },
        { "TLOAD",      loadtga     },
        { "TRANSFORM",  transform   },
        { "TSAVE",      savetga     },
        { "UNDO",       undo        },
        { NULL,         NULL        } }
#endif
    ;

GLOBAL struct {
    char *name;
    int (*loadf)(int, argument *),
        (*savef)(int, argument *);
} formats[]

#ifdef MAIN_MODULE
    = { { "TARGA",      loadtga,    savetga     },
        { "GIF",        loadgif,    savegif     },
        { "RIX",        loadrix,    saverix     },
        { "RAW",        loadraw,    saveraw     },
        { "IP",         loadip,     saveip      },
        { NULL,         NULL,       NULL        } }
#endif
    ;

GLOBAL struct {
    char *name;
	U16 width, height, colors;
	int colorflag;
    int (*initfunc)(void);
    void (*linefunc)(int, U8 *[3]);
    void (*stopfunc)(void);
} displays[]

#ifdef MAIN_MODULE
= { { "CGA",	320,	200,	2,	0,	cgamode,	cgaline,	cgastop	},
	{ "EGA",	640,	350,	8,	1,	egacmode,	egacline,	egacstop  },
	{ "VGA8",	640,	480,	8,	1,	vgacmode,	egacline,	egacstop  },
//	{ "EGA2",	720,	540,	8,	1,	svgamode,	svgaline,	egacstop  },
	{ "GRAY1",	320,	200,  256,	0,	gray1mode,	gray1line,	graystop  },
	{ "GRAY2",	640,	480,  256,	0,	gray2mode,	gray2line,	graystop  },
    { NULL,		  0,	  0,	0,	0,	NULL,		NULL,		NULL	} }
#endif
    ;

GLOBAL char *printers[]
#ifdef MAIN_MODULE
    = { "PAINTJET", "LASERJET", NULL }
#endif
    ;

GLOBAL U8   lookup[3][256], gcmap[256][3];
GLOBAL U32  hist[3][256],
            minalloc            INIT(100000L);
GLOBAL int  leavetemps          INIT(0),
            bleed               INIT(70),
            laserdpi            INIT(300),
            histvalid           INIT(0),
            transpend           INIT(0),
            multiimage          INIT(0),
            multimap            INIT(0);
GLOBAL U16  quitflag            INIT(0),
            interactive         INIT(0),
            xorigin             INIT(0),
            yorigin             INIT(0),
            palette             INIT(8),
            crez                INIT(8);
GLOBAL char printer[STRSIZE]    INIT("LASERJET"),
            printfile[STRSIZE]  INIT("/DEV/PRN"),
            picdir[STRSIZE]     INIT(""),
            fileformat[STRSIZE] INIT("TARGA"),
            display[STRSIZE]    INIT("CGA"),
            tempdir[STRSIZE],
            version[]           INIT("1.10");

GLOBAL struct _vars {       /* This table is used by the SET command.  See  */
    char *varname;          /* SETVAR.C for more details.                   */
    char *varaddr;
    char vartype;
} vartable[]

#ifdef MAIN_MODULE
    = { { "XORIGIN",    (char*)&xorigin,    UTYPE },
        { "YORIGIN",    (char*)&yorigin,    UTYPE },
        { "PALETTE",    (char*)&palette,    UTYPE },
        { "CREZ",       (char*)&crez,       UTYPE },
        { "DPI",        (char*)&laserdpi,   UTYPE },
        { "MULTIIMAGE", (char*)&multiimage, BTYPE },
        { "MULTIMAP",   (char*)&multimap,   BTYPE },
        { "DISPLAY",    display,            STYPE },
        { "PRINTER",    printer,            STYPE },
        { "PRINTFILE",  printfile,          STYPE },
        { "TEMPDIR",    tempdir,            STYPE },
        { "PICDIR",     picdir,             STYPE },
        { "FILEFORMAT", fileformat,         STYPE },
        { NULL,         NULL,               0     } }
#endif
    ;

GLOBAL char reading[]   INIT("Reading %s...\n"),
            writing[]   INIT("Writing %s...\n"),
            working[]   INIT("Working...\n"),
            done[]      INIT("\rDone.    \n");
